///////////////////////////////////////////////////////////////////////////////
//
//  Authors:    Jeffrey Hanke (hanke@cs)
//              Jason Dang (jaydang@cs)
//              Chris Baker (crbaker@cs)
//
//  Design:	Weiss
//
//  File:       UnionFind.java
//
//  Abstract:   Implements the union-find (disjoint set) ADT 
//
///////////////////////////////////////////////////////////////////////////////
import javax.swing.*;
import java.awt.Graphics;
import java.awt.event.*;
import java.awt.*;

public class AMazer extends JApplet implements ActionListener
{

  private Container contentPane;
  private MazeCanvas mazeCanvas;
  private JPanel buttonPanel;
  private JButton createButton;
  private JButton solveButton;
  private JRadioButton dfs, bfs, ids, astar, best;
  private ButtonGroup searchButtons;
  private JTextField widthText;
  private JTextField heightText;
  private JTextField cyclesText;
  private JTextArea debugText;
  private JLabel widthLabel, heightLabel, searchLabel, cyclesLabel;
  private JPanel prefsPanel;
  private JPanel searchPanel;
  private JPanel dummyPanel1, dummyPanel2, dummyPanel3, dummyPanel4, dummyPanel5, dummyPanel6;
  private JPanel actionPanel1, actionPanel2;
  private JScrollPane mazePane;

  private Timer timer; 

  private JScrollPane debugScroll;

  private int width, height, cycles;

  private Maze maze;
  private MazeSearch searcher;
  private String searchType;

  private static final int DEFAULT_MAZE_WIDTH = 35;
  private static final int DEFAULT_MAZE_HEIGHT = 35;
  private static final int DEFAULT_CYCLES = 0;
  private static final String DEFAULT_SEARCHTYPE = "dfs";
  private static final int TIMER_DELAY = 100;
  private static final int APPLET_WIDTH = 820;
  private static final int DEBUG_TEXT_HEIGHT = 100;
  private static final int MAZE_PANE_DIM = 743;
  private static final int MAX_HEIGHT = 100;
  private static final int MAX_WIDTH = 100;

    // added by H. Kautz
    private JPanel infoPanel;
    private JPanel infoTopPanel;
    private JPanel infoDummyPanel;
    private JPanel infoInfoPanel;
    private JLabel infoLabel;
    private JLabel lengthLabel;
    private JTextField lengthText;
    private JLabel nodesLabel;
    private JTextField nodesText;

  public void init() 
  {

    // Maze drawing area
    maze = new Maze( DEFAULT_MAZE_WIDTH,
                     DEFAULT_MAZE_HEIGHT,
                     DEFAULT_CYCLES );
    mazeCanvas = new MazeCanvas( maze );
    maze.setCanvas( mazeCanvas );
    mazePane = new JScrollPane( mazeCanvas );
    mazePane.setPreferredSize( new Dimension( MAZE_PANE_DIM,
                                              MAZE_PANE_DIM ) );
    searcher = new MazeSearch( maze, "dfs", this );
    searchType = DEFAULT_SEARCHTYPE;


    // Text fields
    widthText = new JTextField( Integer.toString( DEFAULT_MAZE_WIDTH ) );
    heightText = new JTextField( Integer.toString( DEFAULT_MAZE_HEIGHT ) );
    debugText = new JTextArea( "" );
    debugText.setLineWrap( true );

    debugScroll = new JScrollPane( debugText );
    debugScroll.setVerticalScrollBarPolicy( 
                 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
    debugScroll.setPreferredSize( new Dimension( APPLET_WIDTH,
                                  DEBUG_TEXT_HEIGHT ) );
    cyclesText = new JTextField( "0" );


    // Buttons
    actionPanel1 = new JPanel();
    actionPanel2 = new JPanel();
    dummyPanel4 = new JPanel();
    dummyPanel5 = new JPanel();
    actionPanel1.setLayout( new GridLayout( 2,1,2,2 ) );
    actionPanel2.setLayout( new GridLayout( 2,1,2,2 ) );
    createButton = new JButton( "Create" );
    solveButton = new JButton( "Solve" );
    actionPanel1.add( dummyPanel4 );
    actionPanel1.add( createButton );
    actionPanel2.add( dummyPanel5 );
    actionPanel2.add( solveButton );

 
    // Labels
    widthLabel = new JLabel( "Width:" );
    heightLabel = new JLabel( "Height:" );
    searchLabel = new JLabel( "Search:" );
    cyclesLabel = new JLabel( "Cycles:" );


    // Prefs panel
    prefsPanel = new JPanel();
    prefsPanel.setLayout( new GridLayout( 3,2,2,2 ) );
    prefsPanel.add( widthLabel );
    prefsPanel.add( widthText );
    prefsPanel.add( heightLabel );
    prefsPanel.add( heightText );
    prefsPanel.add( cyclesLabel );
    prefsPanel.add( cyclesText );

    // Radio Panel
    dfs = new JRadioButton( "DFS", true );
    bfs = new JRadioButton( "BFS" );
    ids = new JRadioButton( "IDS" );
    best = new JRadioButton( "Best" );
    astar = new JRadioButton( "A*" );
    searchButtons = new ButtonGroup();
    searchButtons.add( dfs );
    searchButtons.add( bfs );
    searchButtons.add( ids );
    searchButtons.add( best );
    searchButtons.add( astar );
    searchPanel = new JPanel();
    searchPanel.setLayout( new GridLayout( 6,1,2,2 ) );
    searchPanel.add( searchLabel );
    searchPanel.add( dfs );
    searchPanel.add( bfs );
    searchPanel.add( ids );
    searchPanel.add( best );
    searchPanel.add( astar );

    // Info panel
    infoPanel = new JPanel();
    infoPanel.setLayout( new GridLayout(2, 1, 2, 2 ) );
    infoTopPanel = new JPanel();
    infoTopPanel.setLayout( new GridLayout(2, 1, 2, 2 ) );
    infoDummyPanel = new JPanel();
    infoInfoPanel = new JPanel();
    infoInfoPanel.setLayout( new GridLayout(2, 2, 2, 2 ) );

    infoLabel = new JLabel("Solution");
    lengthLabel = new JLabel("Length:");
    lengthText = new JTextField(" ");
    nodesLabel = new JLabel("Nodes:");
    nodesText = new JTextField(" ");
    
    infoInfoPanel.add( lengthLabel );
    infoInfoPanel.add( lengthText );
    infoInfoPanel.add( nodesLabel );
    infoInfoPanel.add( nodesText );

    infoTopPanel.add( infoDummyPanel );
    infoTopPanel.add( infoLabel );
    infoPanel.add( infoTopPanel );
    infoPanel.add( infoInfoPanel );


    // Buttons panel
    dummyPanel1 = new JPanel();
    dummyPanel2 = new JPanel();
    dummyPanel3 = new JPanel();
    dummyPanel6 = new JPanel();
    buttonPanel = new JPanel();
    buttonPanel.setLayout( new GridLayout( 9,1,2,2 ) );
    buttonPanel.add( actionPanel1 );
    buttonPanel.add( prefsPanel );
    //    buttonPanel.add( dummyPanel1 );
    buttonPanel.add( actionPanel2 );
    buttonPanel.add( searchPanel );

    buttonPanel.add( infoPanel );

    //    buttonPanel.add( dummyPanel6 );
    //    buttonPanel.add( dummyPanel2 );
    //    buttonPanel.add( dummyPanel3 );


    // Action listenters
    dfs.addActionListener( this );
    bfs.addActionListener( this );
    ids.addActionListener( this );
    best.addActionListener( this );
    astar.addActionListener( this );
    createButton.addActionListener( this );
    solveButton.addActionListener( this );


    // Action commands
    dfs.setActionCommand( "dfs" );
    bfs.setActionCommand( "bfs" );
    ids.setActionCommand( "ids" );
    best.setActionCommand( "best" );
    astar.setActionCommand( "astar" );

    solveButton.setActionCommand( "solveButton" ); 
    createButton.setActionCommand( "createButton" );



    // Timer
    timer = new Timer( TIMER_DELAY, this );

    // Applet
    contentPane = this.getContentPane();
    contentPane.setLayout( new BorderLayout() );
    contentPane.add( buttonPanel, BorderLayout.EAST );
    contentPane.add( mazePane, BorderLayout.WEST );
    contentPane.add( debugScroll, BorderLayout.SOUTH );

    mazePane.validate();

    repaint();
  }

  public void actionPerformed( ActionEvent e )
  {

    Object o = e.getSource();

    if( o == dfs 
        || o == bfs
        || o == ids 
        || o == best
        || o == astar )
    {
      solveButton.setText( "Solve" );
      searcher.stopRunning();
      searchType = e.getActionCommand();
      try { Thread.sleep( TIMER_DELAY ); } catch( Exception ex ) {}
      maze.reInitRooms();
    }
    else if( o == createButton )
    {
 
      solveButton.setText( "Solve" );

      // Get new width, height, and cycles
      try
      {
        width = Integer.parseInt( widthText.getText() );
        height = Integer.parseInt( heightText.getText() );
        cycles = Integer.parseInt( cyclesText.getText() );
      }
      catch( NumberFormatException nfe ) 
      { 
        width = 0;
        height = 0;
        cycles = 0;
      }

      // Check for min/max dimensions
      width = (width<2)?2:
              (width>MAX_WIDTH)?MAX_WIDTH:width;
      height = (height<2)?2:
               (height>MAX_HEIGHT)?MAX_HEIGHT:height;
      cycles = (cycles<0)?0:
               (cycles>(width*height-width-height+1))?
                 (width*height-width-height+1):cycles;
      widthText.setText( Integer.toString( width ) );
      heightText.setText( Integer.toString( height ) );
      cyclesText.setText( Integer.toString( cycles ) );

      printText( "******** Created "
                 + width + "x" + height
                 + " Maze (with " + cycles 
                 + " cycles) ********\n\n" );
 
      searcher.stopRunning();

      lengthText.setText("");
      nodesText.setText("");

      mazePane.getViewport().remove( mazeCanvas );

      maze = new Maze( width, height, cycles );
      mazeCanvas = new MazeCanvas( maze );
      maze.setCanvas( mazeCanvas );

      mazePane.getViewport().setView( mazeCanvas );

      mazePane.validate();
      repaint();
    }
    else if( o == solveButton )
    {
      if( solveButton.getText().equals( "Solve" ) )
      {
        // Run search
        if( searchType.equals( "dfs" ) )
          searcher = new MazeSearch( maze, "dfs", this );
        else if( searchType.equals( "bfs" ) )
          searcher = new MazeSearch( maze, "bfs", this );
        else if( searchType.equals( "ids" ) )
          searcher = new MazeSearch( maze, "ids", this );
        else if( searchType.equals( "best" ) )
          searcher = new MazeSearch( maze, "best", this );
        else
          searcher = new MazeSearch( maze, "astar", this );
        solveButton.setText( "Clear");
        searcher.start();
      }
      else
      {
        solveButton.setText( "Solve" );
        searcher.stopRunning();
        try { Thread.sleep( TIMER_DELAY ); } catch( Exception ex ) {}
        maze.reInitRooms();
      lengthText.setText("");
      nodesText.setText("");
      }
    }
    mazeCanvas.repaint();
  }

  public void printText( String text )
  {
    debugText.append( text + "\n\n" );
  }

  public void start() 
  {
    repaint();
    timer.start();
  }

  public void stop() 
  {
    timer.stop();
  }

  public void destroy() 
  {
    timer.stop();
    if( null != searcher )
      searcher.stopRunning();
  }

  public void paintComponent( Graphics g )
  {
    mazeCanvas.paintComponent( g );
  }

    public void printStats(int pathLen, int nodesExp)
    {
        lengthText.setText( Integer.toString( pathLen ) );
        nodesText.setText( Integer.toString( nodesExp ) );
    }
}

